"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateModelObjectToSave = exports.generateParsePropertyAssignments = exports.parseArrayValues = exports.parseValue = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const codegen_ui_1 = require("@aws-amplify/codegen-ui");
const typescript_1 = require("typescript");
const form_state_1 = require("./form-state");
/**
 * JSON.parse(s)
 */
const parseValue = (expression) => typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('JSON'), typescript_1.factory.createIdentifier('parse')), undefined, [expression]);
exports.parseValue = parseValue;
/**
 * modelFields.nonModelFieldArray.map(s => JSON.parse(item))
 */
const parseArrayValues = (accessName) => {
    return typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(accessName, typescript_1.factory.createIdentifier('map')), undefined, [
        typescript_1.factory.createArrowFunction(undefined, undefined, [
            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('s'), undefined, undefined),
        ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), (0, exports.parseValue)(typescript_1.factory.createIdentifier('s'))),
    ]);
};
exports.parseArrayValues = parseArrayValues;
/**
 * arrayFields = nonModelFieldArray: modelFields.nonModelFieldArray.map(s => JSON.parse(item))
 * singleFields =
 *  nonModelField: modelFields.nonModelField ? JSON.parse(modelFields.nonModelField) : modelFields.nonModelField
 */
const generateParsePropertyAssignments = (arrayFields, nonArrayFields, modelFieldsObjectName) => {
    const parseArrayFields = arrayFields.map((field) => typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier(field), (0, exports.parseArrayValues)(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(modelFieldsObjectName), typescript_1.factory.createIdentifier(field)))));
    const parseFields = nonArrayFields.map((field) => typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier(field), typescript_1.factory.createConditionalExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(modelFieldsObjectName), typescript_1.factory.createIdentifier(field)), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), (0, exports.parseValue)(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(modelFieldsObjectName), typescript_1.factory.createIdentifier(field))), typescript_1.factory.createToken(typescript_1.SyntaxKind.ColonToken), typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(modelFieldsObjectName), typescript_1.factory.createIdentifier(field)))));
    return [...parseArrayFields, ...parseFields];
};
exports.generateParsePropertyAssignments = generateParsePropertyAssignments;
const generateModelObjectToSave = (fieldConfigs, modelFieldsObjectName, models, isGraphQL, formActionType) => {
    const nonModelFields = [];
    const nonModelArrayFields = [];
    const inheritFromModelFieldsPropertyAssignments = [];
    let isDifferentFromModelObject = false;
    Object.entries(fieldConfigs).forEach(([name, { validationRules, dataType, sanitizedFieldName, isArray, relationship }]) => {
        const shouldExclude = !dataType || (relationship === null || relationship === void 0 ? void 0 : relationship.type) === 'HAS_MANY';
        const renderedFieldName = sanitizedFieldName || name;
        if (shouldExclude) {
            isDifferentFromModelObject = true;
            return;
        }
        if ((0, codegen_ui_1.isNonModelDataType)(dataType)) {
            if (!isArray) {
                nonModelFields.push(renderedFieldName);
            }
            else {
                nonModelArrayFields.push(renderedFieldName);
            }
            isDifferentFromModelObject = true;
            return;
        }
        if (isGraphQL &&
            ((relationship === null || relationship === void 0 ? void 0 : relationship.type) === 'BELONGS_TO' || (relationship === null || relationship === void 0 ? void 0 : relationship.type) === 'HAS_ONE') &&
            relationship.associatedFields) {
            isDifferentFromModelObject = true;
            const relatedModel = models[relationship.relatedModelName];
            relationship.associatedFields.forEach((associatedFieldName, index) => {
                inheritFromModelFieldsPropertyAssignments.push(typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier(associatedFieldName), formActionType === 'update'
                    ? typescript_1.factory.createBinaryExpression((0, form_state_1.buildAccessChain)([modelFieldsObjectName, renderedFieldName, relatedModel.primaryKeys[index]], true), typescript_1.SyntaxKind.QuestionQuestionToken, typescript_1.factory.createNull())
                    : (0, form_state_1.buildAccessChain)([modelFieldsObjectName, renderedFieldName, relatedModel.primaryKeys[index]], true)));
            });
            return;
        }
        inheritFromModelFieldsPropertyAssignments.push(typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier(name), isGraphQL && formActionType === 'update' && !validationRules.find((r) => r.type === codegen_ui_1.ValidationTypes.REQUIRED)
            ? typescript_1.factory.createBinaryExpression((0, form_state_1.buildAccessChain)([modelFieldsObjectName, name], false), typescript_1.SyntaxKind.QuestionQuestionToken, typescript_1.factory.createNull())
            : (0, form_state_1.buildAccessChain)([modelFieldsObjectName, name], false)));
    });
    const parsePropertyAssignments = (0, exports.generateParsePropertyAssignments)(nonModelArrayFields, nonModelFields, modelFieldsObjectName);
    const modelObjectToSave = typescript_1.factory.createObjectLiteralExpression([...inheritFromModelFieldsPropertyAssignments, ...parsePropertyAssignments], true);
    return { modelObjectToSave, isDifferentFromModelObject };
};
exports.generateModelObjectToSave = generateModelObjectToSave;
//# sourceMappingURL=parse-fields.js.map